(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*                  projet Estime, INRIA-Rocquencourt                  *)
(*                                                                     *)
(*  Copyright 2007-2011 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux <Quentin.Carbonneaux@inria.fr>                 *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: main.ml,v 1.7.2.1 2011-09-21 13:08:04 fclement Exp $ *)

open Ucommand_option;;
open Make_dom2;;
open Parfun;;
open Skl;;
open Skl_extra;;

(* compiler generated code *)

let expor_plot =
  Expor.register (fun (config, rdec) -> Sklstream.map (plot config rdec));;
let plot cfg rdec = atom ((cfg, rdec), expor_plot);;

let expor_subdomain_worker =
  Expor.register
    (fun (config, sd) -> Sklstream.map (subdomain_worker config sd));;
let subdomain_worker cfg sd =
  atom ((cfg, sd), expor_subdomain_worker)
;;

let expor_convergence =
  Expor.register (fun config -> Sklstream.map (convergence config));;
let convergence cfg = atom (cfg, expor_convergence);;

(* end of compiler generated code *)

let add_options () =
(* Module Ucommand_option is provided by CamlP3l, hence the presence here of the
   definition of command-line options. *)
  let annon_fun () =
    Ucommand_option.usage
      (Printf.sprintf
        "%s -- Domain decomposition, solves Poisson's equation"
        Sys.argv.(0));
    exit 2 in
  Ucommand_option.add_user_bi_all [
    ({key = "--number_of_workers";
       spec = Arg.Int set_number_of_workers;
       doc =
         "<nw>  Set the number of workers to <nw>.\
         \n\tDefault value is the number of subdomains, nx * ny, \
         \n\tas provided by the Rdec2 configuration file."},
     Arg.Int ignore),
     "-nw";

    ({key = "--configuration_directory";
       spec = Arg.String set_config_dir;
       doc =
         Printf.sprintf
           "<dir>  Set the configuration directory to <dir>.\
           \n\tDefault directory is %S."
           (get_config_dir ())},
     Arg.String ignore),
    "-cd";

    ({key = "--configuration_filename";
       spec = Arg.String set_config_filename;
       doc =
         Printf.sprintf
           "<fname>  Set the configuration filename to <fname>.\
           \n\tDefault file is %S."
           (get_config_filename ())},
     Arg.String ignore),
    "-cf";

    ({key = "--help";
       spec = Arg.Unit annon_fun;
       doc = "Displays this message list of options.";},
     Arg.Unit annon_fun),
    "-h";
  ];
  Ucommand_option.add_advanced
    (Plain
      (User
        ({ key = "-help"; spec = Arg.Unit annon_fun;
           doc = "Displays this message list of options.";
         }, Arg.Unit annon_fun)))
;;

let generate_input_stream rdec = Sklstream.singleton (set_data rdec)

let skl (cfg_fname, cfg, rdec) =
  let nw = get_number_of_workers () in
  let mk_worker i =
    (subdomain_worker cfg (rdec, i), i :: Rdec2.neighbors_of (rdec, i)) in
  let spec = Array.init nw mk_worker in
  Make_domain.make_domain_opt spec (plot cfg rdec ||| convergence cfg)
;;

let finalize cfg cfg_fname = print_result cfg cfg_fname;;

let main { parfun = parfun; } =
  Ucommand_option.parse_user ();
  let (cfg_fname, cfg, rdec) as gp = get_global_parameters () in
  let is = generate_input_stream rdec in
  let os = parfun (skl gp) is in
  Sklstream.iter ignore os;
  finalize cfg cfg_fname
;;

pardo begin
  add_options ();
  main
end;;
