(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*                  projet Estime, INRIA-Rocquencourt                  *)
(*                                                                     *)
(*  Copyright 2007-2011 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux <Quentin.Carbonneaux@inria.fr>                 *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: make_dom2.ml,v 1.3.2.2 2011-09-21 13:08:04 fclement Exp $ *)

open Config;;
open Rdec2;;
open Gnuplot;;
open My_data;;

(* Management of the user options *)

let get_number_of_workers, set_number_of_workers =
  Uaref.make_ref_single "number_of_workers" 0

and get_config_dir, set_config_dir =
  Uaref.make_ref_single "config_dir" (Filename.concat "." "config_files")

and get_config_filename, set_config_filename =
  Uaref.make_ref_single "config_filename" "config_default"
;;

let get_global_parameters () =
  let cfg_fname =
    Filename.concat (get_config_dir ()) (get_config_filename ()) in
  let cfg = parse_configuration_file cfg_fname in
  let rdec_fname =
    Filename.concat (get_config_dir ()) cfg.rdec_config_filename in
  let rdec = parse_rdec_config_file rdec_fname in
  check_regular_decomposition rdec;
  check_graphical_output cfg.graphical_output;
  if get_number_of_workers () = 0 then begin
    let nsbdx, nsbdy = number_of_subdomains_of_regular_decomposition rdec in
    set_number_of_workers (nsbdx * nsbdy) end;
  cfg_fname, cfg, rdec
;;

(* The elements of the main data stream are of type subdomain array
   (see My_data). *)
let set_data rdec =
  let intf0 = make_zero_interface rdec in
  let f i bd = {
    bd = bd;
    err = max_float, max_float
  } in
  Array.mapi f intf0
;;

let init_poisson sd jtermax (_, oc) =
  let g = computational_rectangular_grid_of_subdomain sd in
  print_computational_rectangular_grid oc g;
  Printf.fprintf oc "%d\n" jtermax;
  flush oc
;;

let subdomain_worker cfg ((rd, i) as sd) =
  let bin = ref None and cout = ref None and iter = ref 0 in
  (fun l ->
    let res = Filename.concat cfg.result_dir cfg.result_basename in
    let inmax = cfg.inner_itermax in
    let w_cmd = Filename.concat cfg.worker_dir cfg.worker_exe in
    let ib, oc =
      Uunix.spawn_init (* FIXME why here ??? *)
        (init_poisson sd inmax) w_cmd bin cout in
    let neighbors = List.map (fun (i, sd) -> (i, sd.bd)) (List.tl l) in
    let cur_bd =
      make_zero_boundary (size_of_boundary (snd (List.hd l)).bd) in
    let bd = normalize_boundary (
      List.fold_left (proj_boundary sd) cur_bd neighbors
    ) in
    let go =
      match cfg.graphical_output with
      | No -> "."
      | Window | Postscript ->
        if !iter mod cfg.display_step <> 0 then "." else
        Printf.sprintf "%s%i" res i in
    Printf.fprintf oc "%s\n" go;
    print_boundary oc bd;
    flush oc;
    let err =
      Scanf.bscanf ib " %f %f\n" (fun e1 e2 -> (e1, e2)) in
    let bd = read_boundary ib (size_of_boundary bd) in
    let res = { bd = bd; err = err; } in
    incr iter; (res, res))
;;

let plot cfg rdec =
  let bin = ref None and cout = ref None and iter = ref 0 in
  (fun x ->
    let go = cfg.graphical_output in
    let bname = Filename.concat cfg.result_dir cfg.result_basename in
    begin match go with
    | No -> ()
    | Window | Postscript ->
      let _ic, oc = Uunix.spawn "gnuplot" bin cout in
      if !iter mod cfg.display_step = 0 then
        let nsbd = number_of_subdomains_of_regular_decomposition rdec in
        let gpl = make_plot_command go bname !iter nsbd in
        Printf.fprintf oc "%s\n" gpl; flush oc
    end;
    incr iter; x)
;;

let convergence cfg =
  let t0 = ref (Unix.gettimeofday ()) and iter = ref 0 in
  (fun v ->
    let f (rs1, re1) {err = rs2, re2; _} = max rs1 rs2, max re1 re2 in
    let rel_stag, rel_err = Array.fold_left f (0. , 0.) v in
    let dt = Unix.gettimeofday () -. !t0 in
    Printf.printf
      "iter = %2d -> rel_stag = %5.0e, rel_err = %6.2f%% (time = %.2f s)\n"
      !iter rel_stag (100. *. rel_err) dt;
    flush stderr;
    (incr iter; !iter) < cfg.outer_itermax)
;;

let print_result cfg fname =
  Printf.printf
    "\nComputation from configuration file %S completed.\n\n" fname;
  match cfg.graphical_output with
  | Window -> Unix.sleep cfg.a_while
  | Postscript | No -> ()
;;
