#######################################################################
#                                                                     #
#                                 Pph                                 #
#                                                                     #
#                    The Portable Project Handler                     #
#                                                                     #
#                  projet Estime, INRIA-Rocquencourt                  #
#                                                                     #
#  Copyright 2005-2009 INRIA.                                         #
#  Institut National de Recherche en Informatique et en Automatique.  #
#  All rights reserved. This file is distributed only by permission.  #
#                                                                     #
#  Francois Clement <Francois.Clement@inria.fr>                       #
#  Pierre Weis <Pierre.Weis@inria.fr>                                 #
#                                                                     #
#######################################################################

# $Id: script_functions.profile,v 1.12 2011-09-12 16:00:54 fclement Exp $

#
# Pph system script functions.
# Used by [un]configure[_user] and pph scripts.
#

SCRIPT=`basename $0`

#
# Error and warning messages.
#

# Generic message printer on stderr.
gen_print_message () {
    $ECHO "$1"
}

print_tagged_message () {
    gen_print_message "$SCRIPT: $1."
}

print_final_tagged_message () {
    $ECHO ""
    print_tagged_message "$1"
    $ECHO ""
}

# To print a typed error message on the standard error.
gen_error () {
    ERROR_TYPE="$1"
    ERROR_MESSAGE="$2"
    
    print_final_tagged_message "$ERROR_TYPE: $ERROR_MESSAGE" 1>&2
}

# To print an error message.
error () {
    gen_error "error" "$1"
}

# To print an internal error message.
internal_error () {
    gen_error "internal error" "$1. Please report"
}

# To print a remedy message.
remedy () {
    if test -n "$1"; then
        gen_print_message "$1" 1>&2
    else
        gen_print_message \
"For help, run
	$SCRIPT --help
" 1>&2
    fi
}

# To print a warning message on the standard error.
warning () {
    print_final_tagged_message "warning: $1" 1>&2
}

#
# Debug, report, see-you and normal operating modes.
#
# mode    | echo message | echo command | run command
# ---------------------------------------------------
# debug   | yes          | yes          | yes
# report  | yes          | no           | yes
# see_you | yes          | yes          | no
# normal  | no           | no           | yes
#

# To print a report message when appropriate on the standard output.
report_message () {
    MESSAGE="$1"
    
    # The use of the boolean operator -o does seem to work everywhere...
    if $DEBUG; then
        print_tagged_message "$MESSAGE"
    elif $REPORT; then
        print_tagged_message "$MESSAGE"
    elif $SEE_YOU; then
        print_tagged_message "$MESSAGE"
    fi
}

# Report a message and append it to a log file.
report_and_log_message () {
    MESSAGE="$1"
    LOG_FILE="$2"
    
    report_message "$MESSAGE" | $TEE_APPEND "$LOG_FILE"
}
echo_and_log_message () {
    MESSAGE="$1"
    LOG_FILE="$2"
    
    $ECHO "$MESSAGE" | $TEE_APPEND "$LOG_FILE"
}

# To run and/or echo a command when appropriate.
run_command () {
    CMD="$1"
    
    # The use of the boolean operator -o does seem to work everywhere...
    if $DEBUG; then
        $ECHO "$CMD"
    elif $SEE_YOU; then
        $ECHO "$CMD"
    fi
    
    if ! $SEE_YOU; then
        eval "$CMD"
        IER=$?
        if test $IER -ne 0; then exit $IER; fi
    fi
}

# To run a command and hide its output.
run_command_hide_output () {
    CMD="$1"
    
    run_command "$CMD" 1> /dev/null 2>&1
}

#
# Warning messages in automatically generated files.
#

gen_newline () {
    WHOS_FOR="$1"
    
    case $WHOS_FOR in
    ( for_sed )
        NEWLINE="\n\\
"
        ;;
    ( * )
        NEWLINE="
"
        ;;
    esac
}

AUTO_MSG="Automatically generated by Pph."

gen_dont_warning_msg_for_sed () {
    SOURCE_FILE="$1"
    
    if test -n "$SOURCE_FILE"; then
        RATHER_MSG=", but rather its source: \"$SOURCE_FILE\""
    else
        RATHER_MSG=""
    fi
    
    MSG="# $AUTO_MSG\n\
#\n\
# Do not edit this file$RATHER_MSG."
    
    $ECHO "$MSG"
}

gen_dont_warning_msg_for_echo () {
    SOURCE_FILE="$1"
    
    if test -n "$SOURCE_FILE"; then
        RATHER_MSG=", but rather its source: \"$SOURCE_FILE\""
    else
        RATHER_MSG=""
    fi
    
    MSG="# $AUTO_MSG
#
# Do not edit this file$RATHER_MSG.

"
    
    $ECHO "$MSG"
}

gen_dont_warning_msg_for_tex () {
    SOURCE_FILE="$1"
    
    if test -n "$SOURCE_FILE"; then
        RATHER_MSG=", but rather its source: \"$SOURCE_FILE\""
    else
        RATHER_MSG=""
    fi
    
    MSG="% $AUTO_MSG
%
% Do not edit this file$RATHER_MSG.

"
    
    $ECHO "$MSG"
}

gen_do_warning_msg_for_echo () {
    MSG="# $AUTO_MSG
#
# Feel free to modify the value of the variable(s), then run configure.
# Remember that each variable must be assigned only once.
# Note also that values must be enclosed between double quotes (\"),
# since we want to allow white spaces in values.
"
    
    $ECHO "$MSG"
}

#
# Files operations.
#

# Copy source file into target only when the target does not exist.
cautious_copy () {
    SRC="$1"
    TGT="$2"
    if test ! -f $TGT; then
        $CP $SRC $TGT
    fi
}

# Get file type.
get_file_type () {
    FILE="$1"
    
    if test -f "$FILE"; then
        FILE_TYPE="regular_file"
    elif test -d "$FILE"; then
        FILE_TYPE="directory"
    elif test -h "$FILE"; then
        FILE_TYPE="symbolic_link"
    else
        FILE_TYPE="other"
    fi
    $ECHO "$FILE_TYPE"
}

# Print file contents.
print_file_contents () {
    FILE="$1"
    
    FILE_TYPE=`get_file_type "$FILE"`
    case $FILE_TYPE in
        ( regular_file ) $CAT $FILE;;
        ( directory ) $LS $FILE;;
        ( symbolic_link ) $ECHO " -> `$LL $FILE | $AWK '{print $NF}'`";;
        ( other ) $ECHO "  *** Cannot print info about \"$FILE\" ***";;
    esac
}

# Filter out the output of a quiet recursive diff.
grep_differ () {
    LIST="$1"
    SEP="$2"
    
    $ECHO "$LIST" | $GREP differ | \
    $AWK '{print $2}' | $AWK -F$SEP '{print $2}'
}

grep_only () {
    LIST="$1"
    SEP="$2"
    
    $ECHO "$LIST" | $GREP "Only in $SEP" | \
    $SED -e 's@: @/@' | $AWK -F$SEP '{print $2}'
}

filter_out_cvs_id_and_banner () {
    FILE1="$1"
    FILE2="$2"
    
    EMPTY_STRING=""
    
    $DIFF $FILE1 $FILE2 | \
    $EGREPV '^---$' | \
    $EGREPV "\\\$I${EMPTY_STRING}d: .*Exp \\\$" | \
    $EGREPV '^> $' | \
    $EGREPV '^> #.*#$' | \
    $EGREPV '^> %.*%$' | \
    $EGREPV '^> <!--.*-->$' | \
    $SED -e 's/^\([134]\)c\1,[1-9][0-9]*$//' | \
    $SED -e 's/^\([1-9][0-9]*\)c\1$//'
}

print_list () {
    LIST="$1"
    
    if test -n "`$ECHO $LIST | $STRIP_SPACE`"; then
        $ECHO ""
        for ITEM in $LIST; do
            $ECHO "    $ITEM"
        done
    fi
}

#
# Strings.
#

capitalize () {
    $ECHO "\$(capitalize \"$1\")" | $HTMLC_FILTER
}

lowercase () {
    $ECHO "\$(lowercase \"$1\")" | $HTMLC_FILTER
}

uppercase () {
    $ECHO "\$(uppercase \"$1\")" | $HTMLC_FILTER
}

current_iso_date () {
    $ECHO '$(the_full_iso_date)' | $HTMLC_FILTER
}

strip_last_suffix () {
    SUFFIX="$1"
    
    $TR -d "$SUFFIX"
}

#
# Variables.
#

# To return the value only when the variable was previously empty.
single_assignment () {
    VARIABLE="$1"
    VALUE="$2"
    if test -n "$VARIABLE"; then
        exit $ALREADY_SET_ERROR
    fi
    $ECHO "$VALUE"
}

# To check that the value of a variable is nonempty.
verify_nonempty () {
    if test -z "$2"; then
        error "$1 must be nonempty"
        exit $EMPTY_VALUE_ERROR
    fi
}

# To check that the value of a variable is a valid Pph file_name.
verify_valid_file_name () {
    if test -z "`$ECHO \"$2\" | $AWK '/^[A-Za-z0-9_\.\-+]+$/{print $0}'`"; then
        error "invalid $1 \"$2\".
Valid characters in Pph file_names are [A-Za-z0-9_\.\-+]"
        exit $INVALID_ARGUMENT_ERROR
    fi
}

# To check that a directory is nonexistant.
verify_new_directory () {
    if test -d "$1"; then
        error "directory $PROJECT_DIRECTORY_NAME exists already"
        exit $INVALID_VALUE_ERROR
    fi
}

# To check that the value of a variable is in a valid range.
validate () {
    eval "
    case \"$3\" in
    $1 ) ;;
    \"\" )
        error \"empty $2\"
        exit $EMPTY_VALUE_ERROR
        ;;
    * )
        error \"unknown $2 \\\"$3\\\"\"
        exit $INVALID_VALUE_ERROR
        ;;
    esac
    "
}

# To expand any variables in the expression.
expand () {
    eval '$ECHO "$1"'
}

# To expand existing file_names in expressions.
expand_file_names () {
    eval $LD $@
}

# To get an absolute path to a given existent directory.
absolute_directory_path () {
    (\cd `expand "$1"` > /dev/null && $ECHO $PWD)
}

# Caml bindings of the following form:
# <let> <space> <ident> <space> <equal> <space> <value> <semisemi>
get_caml_binding_from_file () {
    FILE="$1"
    IDENT="$2"
    
    $CAT $FILE | \
    $EGREP "^let $IDENT = .*;;\$" | \
    $AWK -F" = " '{print $2}' | \
    strip_last_suffix ";;"
}

replace_caml_binding_in_file () {
    FILE="$1"
    IDENT="$2"
    VALUE="$3"
    
    $CAT $FILE | \
    $SED -e "/^let $IDENT = .*;;\$/s/ = .*;;/ = $VALUE;;/" \
      > $FILE.tmp
    $MV $FILE.tmp $FILE
}

#
# Files.
#

# Creation of symbolic links.

create_link () {
    SUB_DIR="$1"
    SOURCE="$2"
    TARGET="$3"
    
    (\cd "$SUB_DIR" > /dev/null && \
     $RM "$TARGET" && \
     $LN "$SOURCE" "$TARGET")
}

safe_create_link () {
    SUB_DIR="$1"
    SOURCE="$2"
    TARGET="$3"
    
    (\cd "$SUB_DIR" > /dev/null
    if test ! -e "$TARGET"; then
        $LN "$SOURCE" "$TARGET"
    elif test ! -h "$TARGET"; then
        FILE="file \"$SUB_DIR/$TARGET\""
        error "$FILE is already existing but it is not a symbolic link."
        exit $WRONG_FILE_TYPE_ERROR
    else
        LNK=`$LLD "$TARGET" | $AWK -F"->" '{print $2}' | $STRIP_SPACE`
        if test "$LNK" != "$SOURCE"; then
            FILE="file \"$SUB_DIR/$TARGET\""
            error "$FILE is a symbolic link towards \"$LNK\"" \
                  "instead of \"$SOURCE\"."
            exit $WRONG_LINK_ERROR
        fi
    fi)
}

# Replace file only if it is different.
replace_if_new () {
    FILE1="$1"
    FILE2="$2"
    TO_DO="$MV \"$FILE1\" \"$FILE2\""
    if test -f "$FILE2"; then
        $CMP "$FILE1" "$FILE2"
        IER=$?
        case $IER in
            0 ) TO_DO="$RM \"$FILE1\"";;
            1 ) ;;
            * ) exit $IER;;
        esac
    fi
    eval "$TO_DO"
}
replace_if_new_ro () {
    FILE1="$1"
    FILE2="$2"
    replace_if_new "$FILE1" "$FILE2"
    $CHMOD_READ_ONLY "$FILE2"
}

# Remove the directory and its parents only if they are empty.
# And dont bother about error messages nor nonzero exit codes.
remove_empty_dir_and_parents () {
    DIR="$1"
    
    $RMDIR_AND_PARENTS $DIR 2> /dev/null || true
}

# Replace the CVS id by a given comment.
strip_cvsid () {
    NEWMSG="$1"
    FILEIN="$2"
    FILEOUT="$3"
    ID="\\\$Id:"
    EXP="Exp \\\$"
    case "$FILEIN" in
        - ) TO_DO="$SED -e 's/$ID .*$EXP/$NEWMSG/'";;
        * ) TO_DO="$SED -e 's/$ID .*$EXP/$NEWMSG/' \"$FILEIN\"";;
    esac
    case "$FILEOUT" in
    - ) eval "$TO_DO";;
    * )
        TMPFILEOUT="$FILEOUT".tmp_strip
        eval "$TO_DO" > "$TMPFILEOUT"
        replace_if_new "$TMPFILEOUT" "$FILEOUT"
        ;;
    esac
}

# Search path for commands.
get_command_search_path () {
    COMMAND_SEARCH_PATH="${PATH}"
    for ROOT_DIR in ${PATH_ROOT_DIRS_INTRINSIC_DEFAULT}; do
        case ${ROOT_DIR}XXX in
        ( */XXX ) ;;
        ( * ) ROOT_DIR=${ROOT_DIR}/;;
        esac
        for SUB_DIR in ${PATH_SUB_DIRS_INTRINSIC_DEFAULT}; do
            DIR=`expand "${ROOT_DIR}${SUB_DIR}"`
            case XXX${COMMAND_SEARCH_PATH}XXX in
            ( XXX${DIR}XXX | XXX${DIR}:* | *:${DIR}XXX | *:${DIR}:* );;
            ( * )
                if test -d ${DIR}; then
                    if test -n "${DIR}"; then
                        COMMAND_SEARCH_PATH="${COMMAND_SEARCH_PATH}:${DIR}"
                    else
                        COMMAND_SEARCH_PATH="${DIR}"
                    fi
                fi
                ;;
            esac
        done
    done
    ${ECHO} ${COMMAND_SEARCH_PATH}
}

# List of absolute paths.
absolute_command_search_path () {
    COMMAND_SEARCH_PATH="$1"
    
    IST=":"
    RES="${IST}"
    for DIR in ${COMMAND_SEARCH_PATH}; do
        ABSOLUTE_DIR=`absolute_directory_path "${DIR}"`
        IER=$?
        if test ${IER} -ne 0; then exit ${IER}; fi
        RES="${RES}${IST}${ABSOLUTE_DIR}"
    done
    ${ECHO} ${RES} | ${AWK} -F${IST}${IST} '{print $2}'
}

# Locate the first executable occurrence of a command (second argument) in the
# list of colon (':') separated directory paths (first argument).
# Returns "not_found" if the command is not found, or not executable.
#
# If you want to modify this, be aware that:
#  * ``which'' and ``whereis'' may fail, do not use the same options
#    everywhere, and may even not be available,
#  * ``type'' do not use the same options everywhere, and may even not be
#    available (e.g. its equivalent in Bourne Shell under OpenBSD is the
#    builtin ``whence'').
locate_command () {
    COMMAND_SEARCH_PATH="$1"
    COMMAND="$2"
    
    # Set the field separator to colon (':').
    IFS=":"
    
    CMD="not_found"
    # case ${COMMAND} in
    # ( /* )
    #     if test -x ${COMMAND}; then
    #         CMD="${COMMAND}"
    #     fi
    #     ;;
    # ( * )
        for DIR in ${COMMAND_SEARCH_PATH}; do
            RES="${DIR}/${COMMAND}"
            if test -x ${RES}; then
                CMD="${RES}"
                break
            fi
        done
    #     ;;
    # esac
    ${ECHO} "${CMD}"
}

# Find the first available executable occurrence of a command in the list
# given as the third argument. The possibly empty second argument allows to
# specify a list of filename extensions that are also searched for (this is
# useful when an optimized version is also available).
# The first argument is the command search path, i.e. a list of colon (':')
# separated directory paths.
find_first_command () {
    COMMAND_SEARCH_PATH="$1"
    EXTS="$2"
    LIST="$3"
    
    if test -z `${ECHO} ${COMMAND_SEARCH_PATH} | ${STRIP_SPACE}`; then
        internal_error "empty command search path"
        exit ${INTERNAL_ERROR}
    fi
    
    if test -z `${ECHO} ${LIST} | ${STRIP_SPACE}`; then
        internal_error "empty command list"
        exit ${INTERNAL_ERROR}
    fi
    
    CMD="None"
    for ITEM in ${LIST}; do
        RES=`locate_command "${COMMAND_SEARCH_PATH}" ${ITEM}`
        case ${RES} in
        ( "not_found" ) ;;
        ( /* )
            if test -n "${EXTS}"; then
                for EXT in ${EXTS}; do
                    if test -x "${RES}${EXT}"; then
                        RES="${RES}${EXT}"
                        break
                    fi
                done
            fi
            CMD="${RES}"
            break
            ;;
        ( * )
            internal_error "\"locate_command\" cannot return \"${RES}\""
            exit ${INTERNAL_ERROR}
            ;;
        esac
    done
    
    if test "${CMD}" = "None"; then
        if test `${ECHO} ${LIST} | ${WC_WORD}` -eq 1; then
            MSG="The command \"${LIST}\" has not been found
in the following path list:"
            REM_MSG="Install it..."
        else
            MSG="None of the commands \"${LIST}\" has been found
in the following path list:"
            REM_MSG="Install one of them..."
        fi
        IFS=":"
        for DIR in ${COMMAND_SEARCH_PATH}; do
            MSG="${MSG}
${DIR}"
        done
        error "${MSG}"
        remedy "${REM_MSG}"
        exit $COMMAND_NOT_FOUND_ERROR
    else
        $ECHO "$CMD"
    fi
}

# To get the value of a given variable from a given configuration file.
#get_variable () {
#    FILE=$1
#    VARIABLE=$2
#    ...
#}

# To record a variable assignment in a given configuration file.
# We use here the makefile syntax.
record_variable_assignment_for_makefile () {
    CONFIGFILE="$1"
    COMMENT="$2"
    VARIABLE="$3"
    VALUE=`expand "$4"`
    $CAT << eof >> "$CONFIGFILE"

# $COMMENT
$VARIABLE = $VALUE
eof
}
# For shell, we need double quotes to allow for white spaces in values.
record_variable_assignment_for_shell () {
    CONFIGFILE="$1"
    COMMENT="$2"
    VARIABLE="$3"
    VALUE=`expand "$4"`
    $CAT << eof >> "$CONFIGFILE"

# $COMMENT.
$VARIABLE="$VALUE"
eof
}
record_variable_assignment_for_htmlc_environment () {
    CONFIGFILE="$1"
    COMMENT="$2"
    VARIABLE="$3"
    VALUE=`expand "$4"`
    $CAT << eof >> "$CONFIGFILE"

(* $COMMENT. *)
let $VARIABLE = "$VALUE";;
eof
}

# To fill a given configuration file with a list of comment/variable/value
# triples.
# An existing file is only regenerated when the contents would be different.
fill_config_file () {
    DIR="$1"
    BASENAME="$2"
    
    FILE_NO_EXTENSION="$DIR/$BASENAME"
    FILE_HTMLC_ENV="${FILE_NO_EXTENSION}.env"
    FILE_MAKEFILE="${FILE_NO_EXTENSION}.pph"
    FILE_SHELL="${FILE_NO_EXTENSION}.profile"
    TMP_FILE_HTMLC_ENV="${FILE_HTMLC_ENV}.tmp_fill"
    TMP_FILE_MAKEFILE="${FILE_MAKEFILE}.tmp_fill"
    TMP_FILE_SHELL="${FILE_SHELL}.tmp_fill"
    MSG_HTMLC_ENV=`gen_do_warning_msg_for_echo`
    MSG_MAKEFILE=`gen_dont_warning_msg_for_echo "${BASENAME}.env"`
    MSG_SHELL=`gen_dont_warning_msg_for_echo "${BASENAME}.env"`
    shift 2
    $ECHO "$MSG_HTMLC_ENV" > "$TMP_FILE_HTMLC_ENV"
    $ECHO "$MSG_MAKEFILE" > "$TMP_FILE_MAKEFILE"
    $ECHO "$MSG_SHELL" > "$TMP_FILE_SHELL"
    while test $# -ge 3; do
        COMMENT="$1"
        VARIABLE="$2"
        VALUE="$3"
        shift 3
        if test "$VALUE" != "$FALSE"; then
            record_variable_assignment_for_htmlc_environment \
                "$TMP_FILE_HTMLC_ENV" "$COMMENT" "$VARIABLE" "$VALUE"
            record_variable_assignment_for_makefile \
                "$TMP_FILE_MAKEFILE" "$COMMENT" "$VARIABLE" "$VALUE"
            record_variable_assignment_for_shell \
                "$TMP_FILE_SHELL" "$COMMENT" "$VARIABLE" "$VALUE"
        fi
    done
    if test $# -ge 1; then
        internal_error "missing value for variable $VARIABLE"
        exit $INTERNAL_ERROR
    fi
    replace_if_new "$TMP_FILE_HTMLC_ENV" "$FILE_HTMLC_ENV"
    replace_if_new_ro "$TMP_FILE_MAKEFILE" "$FILE_MAKEFILE"
    replace_if_new_ro "$TMP_FILE_SHELL" "$FILE_SHELL"
}

# Option documentation.
gen_doc () {
    LONG_OPTION_KEY="$1"
    SHORT_OPTION_KEY="$2"
    ARGUMENT_NAME="$3"
    ARGUMENT_VALUE="<$ARGUMENT_NAME>"
    TEXT="$4"
    
    $ECHO "
$LONG_OPTION_KEY $ARGUMENT_VALUE, \
$SHORT_OPTION_KEY $ARGUMENT_VALUE
		$TEXT."
}
doc () {
    LONG_OPTION_KEY="$1"
    SHORT_OPTION_KEY="$2"
    ARGUMENT_NAME="$3"
    ARGUMENT_VALUE="<$ARGUMENT_NAME>"
    OPTION_NAME="$4"
    
    gen_doc \
        "$LONG_OPTION_KEY" "$SHORT_OPTION_KEY" "$ARGUMENT_NAME" \
        "Set the $OPTION_NAME to $ARGUMENT_VALUE"
}
doc_with_default () {
    LONG_OPTION_KEY="$1"
    SHORT_OPTION_KEY="$2"
    ARGUMENT_NAME="$3"
    ARGUMENT_VALUE="<$ARGUMENT_NAME>"
    OPTION_NAME="$4"
    DEFAULT_VALUE="$5"
    
    gen_doc \
        "$LONG_OPTION_KEY" "$SHORT_OPTION_KEY" "$ARGUMENT_NAME" \
        "Set the $OPTION_NAME to $ARGUMENT_VALUE.
		Default is \"$DEFAULT_VALUE\""
}

# Creating files in the root directory of the project.
create_path_file () {
    PROJECT_SUB_DIR="$1"
    PATH_FILE="$PROJECT_ROOT_DIR/$PROJECT_SUB_DIR/$2"
    
    $RM "$PATH_FILE"
    run_command "$CAT << eof > \"$PATH_FILE\"
PROJECT_ROOT_DIR=$PROJECT_ROOT_DIR
PROJECT_SUB_DIR=$PROJECT_SUB_DIR
eof"
    $CHMOD_READ_ONLY "$PATH_FILE"
}

create_preamble () {
    PREAMBLE_TYPE="$1"
    PREAMBLE_FILE="$PROJECT_ROOT_DIR/$2"
    PREAMBLE_INIT="$3"
    
    $RM "$PREAMBLE_FILE"
    run_command "$CAT << eof > \"$PREAMBLE_FILE\"
include ./$PREAMBLE_INIT
include \\\$(PROJECT_ROOT_DIR)/pph/$PREAMBLE_TYPE/makefiles/Pph_dirs.pph
eof"
    $CHMOD_READ_ONLY "$PREAMBLE_FILE"
}

create_user_preamble () {
    create_preamble "user" "$1" "$2"
}

create_devel_preamble () {
    create_preamble "devel" "$1" "Makefile_user"
}

# Pph pervasive tools.
# Installation immediately follows compilation,
# hence we can use a volatile location for compilation as /tmp.
# Installation goes into $PPH_SYSTEM_ROOT_DIR.
install_pph_pervasive_tool () {
    TOOL="$1"
    
    SRC_DIR="$PPH_SYSTEM_ROOT_DIR/$TOOL"
    TARBALL="$PPH_SYSTEM_ROOT_DIR/tarballs/$TOOL.tgz"
    
    if test -d $SRC_DIR; then
        TO_DO=true
    elif test -f $TARBALL; then
        if test -x $PPH_BIN_DIR/$TOOL; then
            if test $TARBALL -nt $PPH_BIN_DIR/$TOOL; then
                TO_DO=true
            else
                TO_DO=false
            fi
        else
            TO_DO=true
        fi
    else
        error "no sources to build and install \"$TOOL\""
        exit $INVALID_ARGUMENT_ERROR
    fi
    
    if $TO_DO; then
        report_message "Installing Pph pervasive tool \"$TOOL\""
        if test -d $SRC_DIR; then
            (\cd $SRC_DIR && \
             $SH ./build.sh && \
             $SH ./install.sh $PPH_BIN_DIR && \
             $SH ./clean.sh)
        elif test -f $TARBALL; then
            TMP_DIR="/tmp/pph-$USER-$TOOL"
            $RM $TMP_DIR && \
            $MKDIR $TMP_DIR && \
            (\cd $TMP_DIR > /dev/null && \
             $TARXZ $TARBALL && \
             \cd $TOOL-* > /dev/null && \
             ./configure --install-root-dir $PPH_SYSTEM_ROOT_DIR && \
             $MAKE_COMMAND && \
             $MAKE_COMMAND install) 2>&1 && \
            $RM $TMP_DIR
        fi
    else
        report_message \
            "Pph pervasive tool \"$TOOL\" is already installed"
    fi
}

install_pph_pervasive_tools () {
    TOOLS="$1"
    
    if test -n "$TOOLS"; then
        for TOOL in $TOOLS; do
            install_pph_pervasive_tool "$TOOL"
            IER=$?
            if test $IER -ne 0; then exit $IER; fi
        done
    fi
}

uninstall_pph_pervasive_tool () {
    TOOL="$1"
    
    SRC_DIR="$PPH_SYSTEM_ROOT_DIR/$TOOL"
    TARBALL="$PPH_SYSTEM_ROOT_DIR/tarballs/$TOOL.tgz"
    
    if test -d $SRC_DIR; then
        TO_DO=true
    elif test -f $TARBALL; then
        TO_DO=true
    else
        error "no sources uninstall \"$TOOL\""
        exit $INVALID_ARGUMENT_ERROR
    fi
    
    if test ! -x $PPH_BIN_DIR/$TOOL; then
        TO_DO=false
    fi
    
    if $TO_DO; then
        report_message "Uninstalling Pph pervasive tool \"$TOOL\""
        if test -d $SRC_DIR; then
            (\cd $SRC_DIR > /dev/null && \
             $SH ./uninstall.sh $PPH_BIN_DIR) > /dev/null 2>&1
        elif test -f $TARBALL; then
            TMP_DIR="/tmp/pph-$USER-$TOOL"
            $RM $TMP_DIR && \
            $MKDIR $TMP_DIR && \
            (\cd $TMP_DIR > /dev/null && \
             $TARXZ $TARBALL && \
             \cd $TOOL-* > /dev/null && \
             ./configure --install-root-dir $PPH_SYSTEM_ROOT_DIR && \
             $MAKE_COMMAND uninstall) > /dev/null 2>&1 && \
            $RM $TMP_DIR && \
            remove_empty_dir_and_parents $PPH_MAN_DIR && \
            remove_empty_dir_and_parents $PPH_SHR_DIR
        fi
    else
        report_message "Pph pervasive tool \"$TOOL\" is not installed"
    fi
}

uninstall_pph_pervasive_tools () {
    TOOLS="$1"
    
    if test -n "$TOOLS"; then
        for TOOL in $TOOLS; do
            uninstall_pph_pervasive_tool "$TOOL"
            IER=$?
            if test $IER -ne 0; then exit $IER; fi
        done
    fi
}
