(***********************************************************************)
(*                                                                     *)
(*                          Sklml                                      *)
(*                                                                     *)
(*             Pierre Weis (Pierre.Weis@inria.fr)                      *)
(*             Francois Clement (Francois.Clement@inria.fr)            *)
(*             Quentin Carbonneaux (Quentin.Carbonneaux@inria.fr)      *)
(*                                                                     *)
(* This program is free software.                                      *)
(* (see the file LICENSE in the root directory of the distribution).   *)
(*                                                                     *)
(***********************************************************************)

(* $Id: IO1.ml,v 1.13 2011-05-13 12:51:32 qcarbonn Exp $ *)

type in_channel = Pervasives.in_channel;;
type out_channel = Pervasives.out_channel;;

type ic = in_channel;;
type oc = out_channel;;

exception IOError of string;;

let ioerr msg =
  raise (IOError msg)
;;

let read_sockaddr str =
  let sb = Scanf.Scanning.from_string str in
  try
    let (addr, port) =
      Scanf.bscanf sb "%s@:%d"
        (fun a p ->
          (Unix.inet_addr_of_string a, p)) in
    Unix.ADDR_INET (addr, port)
  with _ -> failwith "Invalid address format, use www.xxx.yyy.zzz:port format"
;;

let dump_sockaddr () addr =
  match addr with
  | Unix.ADDR_UNIX addr -> addr
  | Unix.ADDR_INET (addr, port) ->
    Printf.sprintf "%s:%d" (Unix.string_of_inet_addr addr) port
;;

let print_sockaddr oc addr =
  Printf.fprintf oc "%s" (dump_sockaddr () addr)
;;

let stdin = Pervasives.stdin;;
let stdout = Pervasives.stdout;;

let create_socket () =
  Unix.socket Unix.PF_INET Unix.SOCK_STREAM 0
;;

let get_port addr =
  match addr with
  | Unix.ADDR_INET (_, port) -> port
  | Unix.ADDR_UNIX _ -> raise Not_found
;;

let get_addr addr =
  match addr with
  | Unix.ADDR_INET (a, _) -> a
  | Unix.ADDR_UNIX _ -> raise Not_found
;;

let channels_of_sock sock =
  ( Unix.in_channel_of_descr sock
  , Unix.out_channel_of_descr sock)
;;

let close_in ch =
  close_in ch
;;

let close_out ch =
  close_out_noerr ch
;;

let connect addr =
  let fd = create_socket () in
  try
    Unix.connect fd addr;
    channels_of_sock fd
  with Unix.Unix_error (e, _, _) ->
    Unix.close fd;
    ioerr ("IO.connect: " ^ (Unix.error_message e))
;;

let connect_fd addr =
  let fd = create_socket () in
  try
    Unix.connect fd addr;
    fd
  with Unix.Unix_error (e, _, _) ->
    Unix.close fd;
    ioerr ("IO.connect_fd: " ^ (Unix.error_message e))
;;

let create_bound_socket port =
  let port =
    match port with
    | Some p -> p
    | None -> 0 in
  let sock = create_socket () in
  try
    Unix.bind sock (Unix.ADDR_INET (Unix.inet_addr_any, port));
    Unix.listen sock 1;
    (sock, Unix.getsockname sock)
  with Unix.Unix_error (e, _, _) ->
    Unix.close sock;
    ioerr ("IO.create_bound_socket: " ^ (Unix.error_message e))
;;

let flush chan =
  flush chan
;;

let wait_connection sock closeb =
  let fd, addr = Unix.accept sock in
  if closeb then Unix.close sock else ();
  channels_of_sock fd, (get_addr addr)
;;

let to_channel oc x flushb =
  Marshal.to_channel oc x [];
  if flushb then flush oc else ()
;;

let of_channel ic =
  try Marshal.from_channel ic
  with Failure msg ->
    ioerr ("IO.of_channel: " ^ msg)
;;

let in_channel_of_ic ic = ic;;
let out_channel_of_oc oc = oc;;

(* Process management *)

type pid = int;;

type fork_data =
   | Child
   | Parent of pid
;;

let fork () =
  try
    flush stderr; (* Avoid duplicating output *)
    match Unix.fork () with
    | 0 -> Child
    | pid -> Parent pid
  with Unix.Unix_error (e, _, _) ->
    ioerr ("IO.fork: " ^ (Unix.error_message e))
;;

let open_process_in s =
  try Unix.open_process_in s
  with Unix.Unix_error (e, _, _) ->
    ioerr ("IO.open_process_in: " ^ (Unix.error_message e))
;;

let open_process s =
  try Unix.open_process s
  with Unix.Unix_error (e, _, _) ->
    ioerr ("IO.open_process: " ^ (Unix.error_message e))
;;

let pipe () =
  try
    let (ic, oc) = Unix.pipe () in
    ( Unix.in_channel_of_descr ic
    , Unix.out_channel_of_descr oc)
  with Unix.Unix_error (e, _, _) ->
    ioerr ("IO.safe_pipe: " ^ (Unix.error_message e))
;;

let open_in_out str =
  try
    (* This is the one and only way
       to open a fifo without blocking *)
    let fd = Unix.openfile str [ Unix.O_RDWR; Unix.O_SYNC; ] 0 in
    ( Unix.in_channel_of_descr fd
    , Unix.out_channel_of_descr fd )
  with Sys_error m ->
    ioerr ("IO.open_in_out: " ^ m)
;;

let open_in str =
  try
    open_in_bin str
  with Sys_error m ->
    ioerr ("IO.open_in: " ^ m)
;;

let open_default_prems = 0o640;;

let open_out str =
  try
    open_out_bin str
  with Sys_error m ->
    ioerr ("IO.open_out: " ^ m)
;;

let fifo_default_perms = 0o640;;

let mk_fifo k str =
  try
    Unix.mkfifo str fifo_default_perms;
    k str
  with Unix.Unix_error (e, _, _) ->
    ioerr ("IO.openfifo: " ^ (Unix.error_message e))
;;

let openfifo_in str =
  mk_fifo open_in str
;;

let openfifo_out str =
  mk_fifo open_out str
;;
